SystemOrganization addCategory: #RPackage!

!MethodReference class methodsFor: '*RPackage' stamp: 'stephane.ducasse 7/10/2009 09:27'!
class: aClass selector: aSelector package: aPackage

	^ (self class: aClass selector: aSelector ) package: aPackage ; yourself! !

!MethodReference methodsFor: '*RPackage' stamp: 'stephane.ducasse 6/2/2009 16:02'!
methodClass

	^ self actualClass ! !

!MethodReference methodsFor: '*RPackage' stamp: 'stephane.ducasse 7/8/2009 11:26'!
package
	"we will have to change that in the future since we will not rely on * convention"
	
	^ package! !

!MethodReference methodsFor: '*RPackage' stamp: 'stephane.ducasse 7/8/2009 11:27'!
package: aPackage
	
	package := aPackage! !

!MethodReference methodsFor: '*RPackage' stamp: 'stephane.ducasse 7/22/2009 11:47'!
selector

	^ methodSymbol! !

Object subclass: #RPackage
	instanceVariableNames: 'classes packageName methodDict'
	classVariableNames: 'PackageOrganizerClass'
	poolDictionaries: ''
	category: 'RPackage'!

!RPackage commentStamp: 'TestRunner 12/3/2009 22:52' prior: 0!
RPackage is destined to replace PackageInfo for obvious speed reason. 
A package is a set of classes and a set of method extensions to other packages.
Here is a typical scenario:

P1 
	A1DefinedInP1
	B1DefinedInP1	
	A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
P2
	A2DefinedInP2>>methodDefinedInP2
	B2DefinedInB2	

P3
	A3DefinedInP3	
	A2DefinedInP2>>methodDefinedInP3  (was *P2*)

Some queries

P1 classes 
	-> A1DefinedInP1 B1DefinedInP1
P1 definedClasses 	
	-> A1DefinedInP1 B1DefinedInP1
P1 extensionMethods
	-> A2DefinedInP2>>methodDefinedInP1
P1 extensionClasses
	-> A2DefinedInP2
	
P2 includesMethod: #methodDefinedInP2 ofClass: A2DefinedInP2
	-> true	
P2 includesMethod: #methodDefinedInP1 ofClass: A2DefinedInP2
	-> false
"methodDefinedInP1 is defined on A2 but not in P2"





API points
----------
there is a distinction between removing a method from a package (as used in moving a method from one package to another one) 
and removing a method from the system. 


Implementation
--------------
	Since a class in a package may be extended by other packages and such extensions should not
	be reported as part of the package, the implementation should explicitly knows the methods
	is contained (defined in class of the package as well as the classes defined in other packages). 
	
	For now (and before a complete scenario implementation and full optimization pass)
	the class has the following internal representation
	
	classes				: class defined in the packages (without class of the class extensions)
							we only keep class and not their metaclass in the defined list
	packageName			: name of the package
	methodDict 			: dictionary class (including class extensions) -> selectors.
				
Historic Notes: 
---------------
in previous implementation (47) we stored MethodReference so a method would know its packages.
Now we only keep a list of selectors and class.

					
Todo:
-----

Read method todo.
!

!RPackage class methodsFor: 'initialize' stamp: 'TestRunner 12/3/2009 21:59'!
initialize
	"self initialize"
	self packageOrganizerClass: RPackageOrganizer! !

!RPackage class methodsFor: 'instance creation' stamp: 'TestRunner 12/3/2009 22:00'!
named: aString

	| newPackage |
	newPackage := self new packageName: aString ;yourself.
	PackageOrganizerClass default registerPackage: newPackage.
	^ newPackage! !

!RPackage class methodsFor: 'protected' stamp: 'TestRunner 12/3/2009 21:59'!
packageOrganizerClass

	^ PackageOrganizerClass! !

!RPackage class methodsFor: 'protected' stamp: 'StephaneDucasse 12/3/2009 21:45'!
packageOrganizerClass: aPackageOrganizerClass

	PackageOrganizerClass := aPackageOrganizerClass! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:52'!
addClass: aClass
	"pay attention that aClass move all its current method regarless how they are
	packaged, since we get a class and not a package slice (ie class with method extension)"
	
	|nam |
	nam := aClass name. 
	self addClassDefinitionName: nam.
	aClass selectors 
		do: [:each | self addSelector: each ofClassName: nam ].

! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:53'!
addClassDefinition: aClass
	"only add class definition to the receiver"
	
	self addClassDefinitionName: aClass name
	! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 21:47'!
addClassDefinitionName: aClassName
	"only add class definition to the receiver"
	
	classes add: aClassName.
	PackageOrganizerClass default registerPackage: self forClassName: aClassName
	! !

!RPackage methodsFor: 'compiled method - api' stamp: 'TestRunner 12/3/2009 17:26'!
addMethod: aCompiledMethod
	"Add the method to the receiver. If the class is not locally defined in that package then 
	the method is defined as a method extension: ie extending another package.
	The method is not added to the class."

	
	| mr mthClass |
	mthClass := aCompiledMethod methodClass. 
	(methodDict at:  mthClass name ifAbsentPut: [Set new]) add: aCompiledMethod selector.
	^ aCompiledMethod.! !

!RPackage methodsFor: 'compiled method - api' stamp: 'TestRunner 12/3/2009 10:18'!
addMethodToSystem: aCompiledMethod
	"Add the method to the receiver. If the class is not locally defined in that package then 
	the method is defined as a method extension: ie extending another package."
	"Add also the method to the class"
	
	self addMethod: aCompiledMethod.
	aCompiledMethod methodClass addSelector: aCompiledMethod selector withMethod: aCompiledMethod.
	^ aCompiledMethod 
	! !

!RPackage methodsFor: 'method addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:47'!
addSelector: aSelector ofClass: aClass
	"Add the method to the receiver. If the class is not locally defined in that package then 
	the method is defined as a method extension: ie extending another package.
	The method is not added to the class."

	
	^ self addSelector: aSelector ofClassName: aClass name! !

!RPackage methodsFor: 'method addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:46'!
addSelector: aSelector ofClassName: aClassName
	"Add the method to the receiver. If the class is not locally defined in that package then 
	the method is defined as a method extension: ie extending another package.
	The method is not added to the class."

	
	(methodDict at: aClassName name ifAbsentPut: [Set new]) add: aSelector.
	^ aSelector.! !

!RPackage methodsFor: 'migration helpers' stamp: 'TestRunner 12/3/2009 18:25'!
addToPackageClassesFromCategory: aString

	| addedClasses |
	addedClasses := (self classesFromCategory: aString).
	self classes addAll: ((self classesFromCategory: aString) collect: [:each | each name]).
	^ addedClasses ! !

!RPackage methodsFor: 'migration helpers' stamp: 'stephane.ducasse 5/29/2009 15:27'!
baseCategoryOfMethod: aMethodReference
	| oldCat oldPrefix tokens | 
	oldCat := aMethodReference category.
	({ 'as yet unclassified'. 'all' } includes: oldCat) ifTrue: [ oldCat := '' ].
	tokens := oldCat findTokens: '*-' keep: '*'.

	"Strip off any old prefixes"
	((tokens at: 1 ifAbsent: [ '' ]) = '*') ifTrue: [
		[ ((tokens at: 1 ifAbsent: [ '' ]) = '*') ]
			whileTrue: [ tokens removeFirst ].
		oldPrefix := tokens removeFirst asLowercase.
		[ (tokens at: 1 ifAbsent: [ '' ]) asLowercase = oldPrefix ]
			whileTrue: [ tokens removeFirst ].
	].

	tokens isEmpty ifTrue: [^ 'as yet unclassified'].
	^ String streamContents:
		[ :s |
		tokens
			do: [ :tok | s nextPutAll: tok ]
			separatedBy: [ s nextPut: $- ]]! !

!RPackage methodsFor: 'private' stamp: 'stephane.ducasse 5/29/2009 15:55'!
category: categoryName matches: prefix
	^ categoryName notNil and: [categoryName = prefix or: [categoryName beginsWith: prefix, '-']]! !

!RPackage methodsFor: 'accessing' stamp: 'stephane.ducasse 7/5/2009 16:32'!
classes 
	"Ideally classes should have returned all the classes (defined and extension) 
	but since PackageInfo>>classes only returned defined classes we do the same"
	^ classes! !

!RPackage methodsFor: 'private' stamp: 'stephane.ducasse 5/29/2009 19:19'!
classesFromCategory: aString
	"private methods: not intended to be called by clients. This method is just an helper to 
	support the migration from package info or categories to classes"
	
	^((self systemCategories: aString) 
		gather: [:cat |
					(SystemOrganization listAtCategoryNamed: aString)
						collect: [:className | Smalltalk at: className]])
						sortBy: [:a :b | a className <= b className]! !

!RPackage methodsFor: 'accessing' stamp: 'stephane.ducasse 6/29/2009 08:38'!
definedClasses 
	"return the list of the class defined in a package by opposition to all classes which are the 
	extended classes and the defined ones"
	
	^ classes! !

!RPackage methodsFor: 'accessing' stamp: 'TestRunner 12/3/2009 16:12'!
extensionClasses 
	"return the classes that the package extends in other packages: the classes that participate into the receiver package but for classes defined in other packages."
	
	^ methodDict keys difference: classes 
	! !

!RPackage methodsFor: 'accessing' stamp: 'TestRunner 12/3/2009 16:13'!
extensionMethods
	"extension methods are methods defined on classes that are not defined on the receiver"
	
	| res |
	res := Set new. 
	self extensionClasses do: [:each |
			res addAll: (methodDict at: each)].
		"should break when we have metaclass extensions but not the class..."
	^ res! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/3/2009 17:31'!
extensionMethodsForClass: aClass
	"return all the method extensions associated with the class but from the perspective 
	of the receiver package"
	"probably not needed since methodsForClass: is doing the same job"

	^  self extensionMethodsForClassName: aClass name
	! !

!RPackage methodsFor: 'testing' stamp: 'StephaneDucasse 12/3/2009 15:56'!
extensionMethodsForClassName: aClassName
	"return all the method extensions associated with the class but from the perspective 
	of the receiver package"
	"probably not needed since methodsForClass: is doing the same job"
	
	^  methodDict at: aClassName ifAbsent: [Set new]
	! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/3/2009 16:06'!
includesClass: aClass
	"Returns true whether the class, aClass, is one of the locally defined classes of the receiver. 
	a metaclass belongs to a package if its class belongs to it. "
	
	^ classes includes: aClass theNonMetaClass name! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/3/2009 16:03'!
includesClassNamed: aString

	^ (classes anySatisfy: [:each | each = aString])! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/3/2009 17:31'!
includesMethod: aSelector ofClass: aClass
	"return true if the receiver include the method of selector aSelector. It can be 
	a method of a class defined in the package or an extension method. 
	Methods extended by other package are not considered"
	
	
	| sels |
	sels := methodDict at: aClass name ifAbsent: [^ false].
	^ sels includes: aSelector.! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/3/2009 17:31'!
includesMethod: aSelector ofClassName: aClassName
	"return true if the receiver include the method of selector aSelector. It can be 
	a method of a class defined in the package or an extension method. 
	Methods extended by other package are not considered"
	
	
	| sels |
	sels := methodDict at: aClassName ifAbsent: [^ false].
	^ sels includes: aSelector.! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/3/2009 17:35'!
includesOrTouchesClass: aClass
	"Returns true whether the class, aClass, is one of the locally defined classes of the receiver or 
	if the receiver extends such class (that is defined in another package"
	
	^ methodDict keys includes: aClass name! !

!RPackage methodsFor: 'private' stamp: 'stephane.ducasse 5/29/2009 19:16'!
includesSystemCategory: categoryName

	self flag: #tofix.
	^ self category: categoryName matches: self packageName! !

!RPackage methodsFor: 'initialize-release' stamp: 'StephaneDucasse 12/3/2009 21:47'!
initialize

	super initialize.
	classes := Set new.
	methodDict := Dictionary new. ! !

!RPackage methodsFor: 'package operations' stamp: 'TestRunner 12/3/2009 18:01'!
mergeInto: aTargetPackage
	"merge the receiver into the targetPackage"
	

	self definedClasses do:
		[:each | self moveClassDefinitionName: each to: aTargetPackage ].

	methodDict keysAndValuesDo:  
		[:key :value | 
			value do: [:sel | 
				self moveSelector: sel ofClassName: key to: aTargetPackage]]
	
	
! !

!RPackage methodsFor: 'testing' stamp: 'TestRunner 12/2/2009 21:36'!
methodsForClass: aClass
	"return all the method associated with the class but from the perspective 
	of the receiver package"
	
	
	^  methodDict at: aClass ifAbsent: [Set new]! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 9/11/2009 17:01'!
moveClassDefinition: aClass to: aPackage

	self removeClassDefinition: aClass.
	aPackage addClassDefinition: aClass ! !

!RPackage methodsFor: 'class addition - removal' stamp: 'TestRunner 12/3/2009 17:45'!
moveClassDefinitionName: aClassName to: aPackage

	self removeClassDefinitionName: aClassName.
	aPackage addClassDefinitionName: aClassName! !

!RPackage methodsFor: 'compiled method - api' stamp: 'TestRunner 12/3/2009 10:24'!
moveMethod: aCompiledMethod to: aPackage

	self removeMethod: aCompiledMethod.
	aPackage addMethod: aCompiledMethod.! !

!RPackage methodsFor: 'method addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:50'!
moveSelector: aSelector ofClass: aClass to: aPackage

	self moveSelector: aSelector ofClassName: aClass name to: aPackage! !

!RPackage methodsFor: 'method addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:49'!
moveSelector: aSelector ofClassName: aClassName to: aPackage


	self removeMethod: aSelector ofClassName: aClassName.
	aPackage addSelector: aSelector ofClassName: aClassName.! !

!RPackage methodsFor: 'accessing' stamp: 'StephaneDucasse 12/3/2009 22:47'!
numberOfMethods

	^ methodDict values inject: 0 into: [:sum :each | sum + each size]! !

!RPackage methodsFor: 'accessing' stamp: 'stephane.ducasse 5/29/2009 15:52'!
packageName

	^ packageName! !

!RPackage methodsFor: 'accessing' stamp: 'stephane.ducasse 5/29/2009 15:52'!
packageName: aSymbol

	packageName := aSymbol asSymbol! !

!RPackage methodsFor: 'printing' stamp: 'stephane.ducasse 7/5/2009 17:47'!
printOn: aStream

	super printOn: aStream.
	packageName isNil 
		ifFalse: [aStream nextPutAll: '(', packageName asString, ')']! !

!RPackage methodsFor: 'class addition - removal' stamp: 'TestRunner 12/3/2009 22:07'!
removeClass: aClass
	"remove the class from the package but not from the system"
	
	|nam |
	self removeClassDefinition: aClass theNonMetaClass.
	"we only keep class and not their metaclass in the defined list"
	nam := aClass name. 
	(methodDict keys includes: nam)
		ifTrue: [methodDict removeKey: aClass name].
		"if we only add a class without anymethod then for now 
		we do not add an entry in the method so we cannot remove it"
	^ aClass! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:54'!
removeClassDefinition: aClass
	"remove the class definition from the package but not its method"
	
	self removeClassDefinitionName: aClass name! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 21:47'!
removeClassDefinitionName: aClassName
	"remove the class definition from the package but not its method"
	
	| removed |
	removed := classes remove: aClassName ifAbsent: [nil].
	removed ifNotNil: [PackageOrganizerClass default 
								unregisterPackage: self forClassName: aClassName.] 
	! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 21:48'!
removeClassFromAllPackages: aClass

	self removeClass: aClass.
	(PackageOrganizerClass default packages 
		select: [:each | each includesOrTouchesClass: aClass ])
		do: [:each | each removeClass: aClass]
	"this should not be right since removing a class from all packages
	should certainly 
		remove the class from its home
		remove all the class method extensions"
	! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 9/11/2009 15:36'!
removeClassFromSystem: aClass

	self removeClassFromAllPackages: aClass.
	aClass removeFromSystem.! !

!RPackage methodsFor: 'class addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:53'!
removeClassName: aClassName
	"remove the class from the package but not from the system"
	
	self removeClassDefinitionName: aClassName.
	methodDict removeKey: aClassName ifAbsent: [nil].
	^ aClassName! !

!RPackage methodsFor: 'todo' stamp: 'stephane.ducasse 6/24/2009 11:08'!
removeFromSystem

	self flag: #shouldFixIt.
	
	^ true! !

!RPackage methodsFor: 'compiled method - api' stamp: 'TestRunner 12/3/2009 22:09'!
removeMethod: aCompiledMethod
	"remove method to the receiver but does not propagate to the class."
	
	^ (methodDict at: aCompiledMethod methodClass name ifAbsent: [ Set new]) 
			remove: aCompiledMethod selector.
	! !

!RPackage methodsFor: 'method addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:48'!
removeMethod: aSelector ofClass: aClass
	"remove the method in the package but does not propagate to the class itself."
	
	self removeMethod: aSelector ofClassName: aClass name
	! !

!RPackage methodsFor: 'method addition - removal' stamp: 'StephaneDucasse 12/3/2009 15:47'!
removeMethod: aSelector ofClassName: aClassName
	"remove the method in the package but does not propagate to the class itself."
	
	(methodDict 
		at: aClassName
		ifAbsent: [ ^ Set new]) remove: aSelector.
	! !

!RPackage methodsFor: 'compiled method - api' stamp: 'stephane.ducasse 8/11/2009 22:55'!
removeMethodFromSystem: aMethod
	"remove method to the receiver and from its class too."
	
	self removeMethod: aMethod.
	aMethod methodClass removeSelector: aMethod selector. 
	^ aMethod
	! !

!RPackage methodsFor: 'accessing' stamp: 'TestRunner 12/3/2009 22:25'!
selectors
	"selectors is the set of all the method selectors used in this packages"
	
	^ methodDict values inject: Set new into: [:sum :each | sum addAll: each]! !

!RPackage methodsFor: 'private' stamp: 'stephane.ducasse 5/29/2009 19:18'!
systemCategories: aString

	self flag: #tofix.
	^ SystemOrganization categories select: [:cat | cat = aString]! !

!RPackage methodsFor: 'todo' stamp: 'TestRunner 12/3/2009 22:52'!
todo

	^ '
	- pay attention that in the dictionary we can have classes and metaclasses and not in the 
		defined classes, so doing the differences should break.
		This point may be fixed by the design of RBSelectorEnvironment (having two tables one of instance and one for meta)
	- rename class handling
	- when we remove all the method of a class then we should remove the key from the dictionary methodDict
	- events
	- migrating  from PackageInfo
	
	- support for * importing when we import other code
	- support for when the guy define a method cat with * 
		-> hacker le systemNotification to get events and add or remove
		
	- monticello inchange
	
	- PackageOrganizer should be the factory: it means nobody should refer directly to PackageInfo
	PackageOrganizer should be the emitter of events.
	MCBrowser and friends should listen to them.
	
	 if we have a class extension and finally moved the class to the same place as the extension, it should recategorized.
- class added automatically when created using category: to indicate package  (self flag: #AutoPackageStep.).
- create package on the fly given the category:??
- integration in the browser ad * conversion

- Class should know its package
- May be rename extensionClasses into extendedClasses and add methodsExtendedByOtherPackages

- package for unpackaged in PackageOrganizer
- way to resynchronise RPackageOrganizer with the instance of RPackages

Open Question:
- do we keep category and packages?

- can we store the package for a compiledMethod or a MethodReference
	if (A1>>#foo) is an extension of Package P1
	can we do (A1>>#foo) isClassExtension, (A1>>#foo) package, ....
- what about the event created by the package creation, method move, class move.....

	
	
	
	
	
	
	
	'! !

Object subclass: #RPackageOrganizer
	instanceVariableNames: 'classPackageMapping packages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage'!

RPackageOrganizer class
	instanceVariableNames: 'default'!

RPackageOrganizer class
	instanceVariableNames: 'default'!

RPackageOrganizer subclass: #RMockPackageOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage'!

!RMockPackageOrganizer class methodsFor: 'singleton' stamp: 'stephane.ducasse 8/10/2009 11:35'!
reset

	default := nil! !

!RPackageOrganizer class methodsFor: 'singleton' stamp: 'stephane.ducasse 7/16/2009 23:23'!
default

	default isNil
		ifTrue: [default := self new].
	^ default! !

!RPackageOrganizer methodsFor: 'initialize' stamp: 'TestRunner 12/3/2009 16:51'!
initialize
	
	super initialize.
	packages := Dictionary new.
	classPackageMapping := Dictionary new.! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'stephane.ducasse 7/16/2009 23:17'!
packageNames
	^ packages keys! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'StephaneDucasse 10/27/2009 21:21'!
packageOf: aClass

	self flag: #shouldProbablyReturnUnunpackagedPackage.
	^ classPackageMapping at: aClass ifAbsent: [nil]
	! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'stephane.ducasse 7/10/2009 11:27'!
packages
	
	^ packages values! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'StephaneDucasse 12/3/2009 16:16'!
registerPackage: aPackage
	packages at: aPackage packageName asSymbol put: aPackage.
	! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'StephaneDucasse 12/3/2009 16:17'!
registerPackage: aPackage forClass: aClass 
	(aPackage includesClassNamed: aClass name) 
		ifFalse: 
			[self error: aPackage name , ' does not includes the class ' , aClass name].
	^classPackageMapping at: aClass name asSymbol put: aPackage! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'TestRunner 12/3/2009 21:42'!
registerPackage: aPackage forClassName: aClassNameSymbol
	(aPackage includesClassNamed: aClassNameSymbol asSymbol) 
		ifFalse: 
			[self error: aPackage name , ' does not includes the class ' , aClassNameSymbol].
	^classPackageMapping at: aClassNameSymbol put: aPackage! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'StephaneDucasse 12/3/2009 16:17'!
unregisterClass: aClass 
	
	^ classPackageMapping removeKey: aClass name asSymbol
	! !

!RPackageOrganizer methodsFor: 'accessing' stamp: 'StephaneDucasse 12/3/2009 16:17'!
unregisterPackage: aPackage
	packages removeKey: aPackage packageName asSymbol ifAbsent: [].	
	! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'StephaneDucasse 12/3/2009 16:18'!
unregisterPackage: aPackage forClass: aClass 
	(aPackage includesClassNamed: aClass name asSymbol) 
		ifTrue: 
			[self error: aPackage name , ' still includes the class ' , aClass name asSymbol].
	^classPackageMapping removeKey: aClass name asSymbol! !

!RPackageOrganizer methodsFor: 'package access from class' stamp: 'StephaneDucasse 12/3/2009 16:18'!
unregisterPackage: aPackage forClassName: aClassName
	(aPackage includesClassNamed: aClassName asSymbol) 
		ifTrue: 
			[self error: aPackage name , ' still includes the class ' , aClassName asSymbol].
	^classPackageMapping removeKey: aClassName asSymbol! !

!ClassDescription methodsFor: '*rpackage' stamp: 'sd 10/29/2009 20:31'!
compileSilently: code 
	"Compile the code and classify the resulting method in the given category, leaving no trail in the system log, nor in any change set, nor in the 'recent submissions' list. This should only be used when you know for sure that the compilation will succeed."

	^ self compileSilently: code classified: 'not defined category' notifying: nil.! !

!ClassDescription methodsFor: '*rpackage' stamp: 'TestRunner 12/3/2009 16:14'!
extendingPackages
	"the extending packages of a class are the packages that extend it."
	
	^ RPackageOrganizer default packages select: [:each | each extensionClasses includes: self name]
	"really slow should be changed to optimize"
	! !

!ClassDescription methodsFor: '*rpackage' stamp: 'TestRunner 12/3/2009 21:58'!
package
	
	self flag: #toBefixed.
	^ RPackage packageOrganizerClass default packageOf: self name.
	! !

!ClassDescription methodsFor: '*rpackage' stamp: 'StephaneDucasse 10/28/2009 08:34'!
packages
	"the extending packages of a class are the packages that extend it."
	
	^  self extendingPackages asSet add: self package; yourself! !

!Behavior methodsFor: '*RPackage' stamp: 'stephane.ducasse 5/29/2009 19:30'!
< aClass
	
	aClass isBehavior ifFalse: [^ false].
	^ self name < aClass name! !

!Behavior methodsFor: '*RPackage' stamp: 'stephane.ducasse 5/29/2009 19:29'!
<= aMagnitude 
	"Answer whether the receiver is less than or equal to the argument."

	^(self > aMagnitude) not! !

!Behavior methodsFor: '*RPackage' stamp: 'stephane.ducasse 5/29/2009 19:30'!
> aClass
	"Answer whether the receiver is greater than the argument."

	^aClass < self

	! !

!Behavior methodsFor: '*RPackage' stamp: 'stephane.ducasse 5/29/2009 19:30'!
>= aMagnitude 
	"Answer whether the receiver is greater than or equal to the argument."

	^(self < aMagnitude) not! !

TestCase subclass: #RPackageExtensionTest
	instanceVariableNames: 'classesCreated packagesCreated defaultRepository packageNamepackageName createdClasses createdPackages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage'!

!RPackageExtensionTest class methodsFor: 'history' stamp: 'stephane.ducasse 7/10/2009 09:11'!
lastStoredRun
	^ ((Dictionary new) add: (#passed->((Set new) add: #testClassSorted; add: #testPackageCreation; add: #testPackageAddClassesFromCategory; add: #testPackageClassesFromCategory; yourself)); add: (#timeStamp->'10 July 2009 9:11:38 am' asTimeStamp); add: (#failures->((Set new))); add: (#errors->((Set new))); yourself)! !

!RPackageExtensionTest methodsFor: 'test for method reference' stamp: 'stephane.ducasse 7/8/2009 23:18'!
methodReferencePackageOldTest
	"self debug: #MethodReferencePackage"
	
	
	self shouldnt: [(Character>>#escapeEntities) methodReference package] raise: Error.
	
	self flag: #toBechangedWhenIntegratedInPharo.
	self assert: (MethodReference>>#package) methodReference package name = 'RPackage'.
	
	"if this is not an extension just return the package of the class"
	self assert: (MethodReference>>#actualClass) methodReference package name = 'Kernel-Traits'.
	
	! !

!RPackageExtensionTest methodsFor: 'test for behavior' stamp: 'stephane.ducasse 5/29/2009 19:32'!
testClassSorted
	"self debug: #testClassSorted"
	

	self assert: Point  < Rectangle.
	self assert: Object < Point.
	self assert: Object < Rectangle.
	self deny: Point > Rectangle.
	self deny: Object > Point.
	self deny: Object> Rectangle
	
	 ! !

!RPackageExtensionTest methodsFor: 'tests' stamp: 'stephane.ducasse 8/11/2009 11:35'!
testPackageAddClassesFromCategory
	"self debug: #testPackageAddClassesFromCategory"
	
	| pck classes |
	pck := RPackage named: #RPackageTest.
	classes := pck addToPackageClassesFromCategory: 'RPackage'.
	self assert: classes size = (SystemOrganization classesInCategory: 'RPackage') size.
	self assert: (classes includes: self class). 
	self assert: (pck includesClass: self class).
	self assert: (pck includesClassNamed: self class name).
	! !

!RPackageExtensionTest methodsFor: 'testing private methods' stamp: 'stephane.ducasse 8/11/2009 22:52'!
testPackageClassesFromCategory
	"self debug: #testPackageClassesFromCategory"
	"private methods: not intended to be called by clients. This method is just an helper to 
	support the migration from package info or categories to classes"
	
	| pck classes |
	pck := RPackage named: #RPackageTest.
	classes := pck classesFromCategory: 'RPackage'.
	self assert: classes size = (SystemOrganization classesInCategory: 'RPackage') size..
	self assert: (classes includes: self class). 
	! !

!RPackageExtensionTest methodsFor: 'tests' stamp: 'stephane.ducasse 5/29/2009 19:31'!
testPackageCreation
	"self debug: #testPackageCreation"
	
	| pck |
	pck := RPackage named: #foo.
	self assert: pck packageName = #foo.
	self assert: pck classes isEmpty! !

TestCase subclass: #RPackageTest
	instanceVariableNames: 'defaultRepository createdClasses createdPackages'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage'!

!RPackageTest commentStamp: 'stephane.ducasse 7/10/2009 10:41' prior: 0!
The tests takes as fixture the following situation (which is constructed partially)
For tests that do not need to check the incremental construction but more the evolution
from this situation use the test RPackageDynamicTest since the setup is exactly the described situation.

P1 
	A1DefinedInP1>>methodDefinedInP1
	B1DefinedInP1	
	A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
P2
	A2DefinedInP2>>methodDefinedInP2
	B2DefinedInB2	

P3
	A3DefinedInP3	
	A2DefinedInP2>>methodDefinedInP3  (was *P2*)!

RPackageTest subclass: #RPackageDynamicTest
	instanceVariableNames: 'p1 p2 p3 a1 b1 a2 a3 b2 cleanP1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage'!

!RPackageDynamicTest commentStamp: 'stephane.ducasse 8/12/2009 10:20' prior: 0!
The tests takes as fixture the following situation and exercises the dynamic aspect of packages.
For tests that do not require the full setup please use RPackageTest

P1 
	A1DefinedInP1
	A1DefinedInP1>>methodDefinedInP1
	B1DefinedInP1	
	A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
P2
	A2DefinedInP2
	A2DefinedInP2>>methodDefinedInP2
	B2DefinedInB2	

P3
	A3DefinedInP3	
	A2DefinedInP2>>methodDefinedInP3  (was *P2*)!

!RPackageDynamicTest class methodsFor: 'customize' stamp: 'stephane.ducasse 7/10/2009 10:37'!
shouldInheritSelectors

	^ false! !

!RPackageDynamicTest methodsFor: 'initialize' stamp: 'StephaneDucasse 10/29/2009 18:01'!
createNewClassNamed: aName
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'RPackageTest'.
	^ cls! !

!RPackageDynamicTest methodsFor: 'initialize' stamp: 'StephaneDucasse 10/29/2009 18:14'!
createNewClassNamed: aName inPackage: p
	
	| cls |
	cls := self createNewClassNamed: aName.
	p addClassDefinition: cls.
	^ cls! !

!RPackageDynamicTest methodsFor: 'initialize' stamp: 'StephaneDucasse 10/29/2009 18:01'!
createNewPackageNamed: aName

	^ RPackage named: aName.
	! !

!RPackageDynamicTest methodsFor: 'initialize' stamp: 'TestRunner 12/3/2009 17:26'!
setUp


	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	a1 :=  self createNewClassNamed: #A1DefinedInP1 inPackage: p1.
	b1 := self createNewClassNamed: #B1DefinedInP1 inPackage: p1.
	a2 := self createNewClassNamed: #A2DefinedInP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2DefinedInP2 inPackage: p2.
	a3 := self createNewClassNamed: #A3DefinedInP3 inPackage: p3.
	
	a1 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#methodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).
! !

!RPackageDynamicTest methodsFor: 'initialize' stamp: 'TestRunner 12/3/2009 22:01'!
tearDown

	
	RPackage packageOrganizerClass default unregisterPackage: p1.
	RPackage packageOrganizerClass default unregisterPackage: p2.
	RPackage packageOrganizerClass default unregisterPackage: p3.
	
	(p1 includesClass: a1)
		ifTrue: [ RPackage packageOrganizerClass default unregisterClass: a1].
	(p1 includesClass: b1)
		ifTrue: [ RPackage packageOrganizerClass default unregisterClass: b1].
	(p2 includesClass: a2)
		ifTrue: [ RPackage packageOrganizerClass default unregisterClass: a2].	
	RPackage packageOrganizerClass default unregisterClass: b2.
		
	RPackage packageOrganizerClass default unregisterClass: a3.	
	
	p1 := nil.
	p2 := nil.
	p3 := nil.
	a1 := nil.
	b1 := nil.
	a2 := nil.
	b2 := nil.
	a3 := nil.
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:40'!
testAddMethod
	"self debug: #testAddMethod"
	
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	
	a1 compileSilently: 'zork ^ #methodDefinedInP1'.
	p1 addMethod: (a1>>#zork).
	self assert: (p1 includesMethod: #zork ofClass: a1).
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'TestRunner 12/3/2009 22:08'!
testBasicRemoveClass
	"we remove a class we check that it is not in the package anymore"
	"self debug: #testBasicRemoveClass"	

	|  size |
	
	size := p1 classes size. 
	self assert: size = 2.
	self assert: (p1 includesClass: a1).
	self assert: (p1 includesClass: a1 class).
	p1 removeClass: a1.
	self assert: p1 classes size = (size - 1).
	self assert: (p1 includesClass: b1).
	self assert: (p1 includesClass: b1 class).
	self deny: (p1 includesClass: a1).
	self deny: (p1 includesClass: a1 class).
	p1 removeClass: b1.
	self assert: p1 classes size = (size - 2).
	self deny: (p1 includesClass: b1).
	self deny: (p1 includesClass: b1 class).

! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'TestRunner 12/3/2009 11:19'!
testBasicRemoveClassCleanMethodInternalStructure
	"we remove a class we check that it is not in the package anymore"
	"self debug: #testBasicRemoveClassCleanMethodInternalStructure"	

	| allMthds |
	allMthds := (p1 instVarNamed: 'methodDict') values inject: (Set new) into: [ :sum :each |  sum addAll: each].
	self assert: ((allMthds select: [:each | each  = #methodDefinedInP1]) size = 1).
		"two methods have the same names"
	p1 removeClass: a1.
	self deny: (p1 includesClass: a1).
	self deny: (p1 includesClass: a1 class).
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	
	allMthds := (p1 instVarNamed: 'methodDict') values inject: (Set new) into: [ :sum :each |  sum addAll: each].
	self assert: ((allMthds select: [:each | each  = #methodDefinedInP1]) size = 1).
		"two methods have the same names"
	"of course the other classes and methods are still there"
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 9/11/2009 15:28'!
testBasicRemoveClassEvenIfClassIsJustAClassExtension
	"we remove a class we check that it is not in the package anymore"
	"self debug: #testBasicRemoveClassEvenIfClassIsJustAClassExtension"	

	|  size |
	size := p1 classes size. 
	self assert: size = 2.
	self deny: (p1 includesClass: a2).
	self assert: (p1 includesOrTouchesClass: a2).
	p1 removeClass: a2.
	self deny: (p1 includesClass: a2).
	self deny: (p1 includesOrTouchesClass: a2).
	self assert: (p1 methodsForClass: a2) isEmpty 
	

! !

!RPackageDynamicTest methodsFor: 'test accessing' stamp: 'StephaneDucasse 10/29/2009 17:31'!
testExtendingPackagesOfClass
	"since a class can be extended by several packages, we want the complete list of packages that extend
	the class"
	
	"self debug: #testPackagesOfClass"
	
	"P1 
		A1DefinedInP1>>methodDefinedInP1
		B1DefinedInP1	
		A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
	P2
		A2DefinedInP2>>methodDefinedInP2
		B2DefinedInB2	

	P3
		A3DefinedInP3	
		A2DefinedInP2>>methodDefinedInP3  (was *P2*)"
		
	| packages |
	packages := a2 extendingPackages.
	"a2 is extended by p1 and p3"
	self assert: packages size = 2. 
	self assert: (packages includes: p1).
	self deny: (packages includes: p2).
	self assert: (packages includes: p3).
	
	packages := a1 extendingPackages.
	self assert: packages isEmpty. 
	
	
	
	
	! !

!RPackageDynamicTest methodsFor: 'test accessing' stamp: 'stephane.ducasse 8/11/2009 23:15'!
testExtensionMethodOfClass
	"self debug: #testExtensionMethodOfClass"
	
	"P1 
		A1DefinedInP1>>methodDefinedInP1
		B1DefinedInP1	
		A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
	P2
		A2DefinedInP2>>methodDefinedInP2
		B2DefinedInB2	

	P3
		A3DefinedInP3	
		A2DefinedInP2>>methodDefinedInP3  (was *P2*)"
		
	self assert: a2 methodDict keys size = 3.
	
	"the class contains methods defined by several packages 
	however when looking with a package view we only see the package 
	extension."
		
	self assert: (p2 extensionMethodsForClass: a2) size = 1.
	self assert: (p1 extensionMethodsForClass: a1) size = 1.
	"show that extensionMethodsForClass: also works for methods of class
	defined in the package"
	
	self assert: (p1 extensionMethodsForClass: a2) size = 1.
	self assert: (p3 extensionMethodsForClass: a2) size = 1.
	
	
	
	! !

!RPackageDynamicTest methodsFor: 'test full adding removing' stamp: 'TestRunner 12/3/2009 22:08'!
testFullRemoveMethodRemoveFromClass
	"we remove a method so it is not in the package but also not from the class anymore"
	"self debug: #testFullRemoveMethodRemoveFromClass"
	
	p1 removeMethodFromSystem: a1 >> #methodDefinedInP1.
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	self deny: (a1 selectors includes: #methodDefinedInP1)! !

!RPackageDynamicTest methodsFor: 'test merging' stamp: 'TestRunner 12/3/2009 22:53'!
testMergePackage
	"self debug: #testMergePackage"
	
	| methodNumber1 classNumber1 methodNumber2 classNumber2 |
	
	methodNumber1 := p1 numberOfMethods.
	classNumber1 := p1 definedClasses size.
	methodNumber2 := p2 numberOfMethods.
	classNumber2 := p2 definedClasses size.
	
	self assert: (p2 includesClass: a2).
	self assert: (p2 includesClass: b2).
	p2 mergeInto: p1. 
	self assert: (p2 definedClasses isEmpty).
	self assert: p1 definedClasses size = (classNumber1 + classNumber2).
	self assert: p1 numberOfMethods = (methodNumber1 + methodNumber2).
	self assert: (p2 numberOfMethods isZero).
	self assert: (p1 includesClass: a2).
	self assert: (p1 includesClass: b2).
	self assert: (p1 includesClass: a1).
	self assert: (p1 includesClass: b1).
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self assert: (p3 includesOrTouchesClass: a2).
	
	self assert: (p3 extensionClasses includes: a2 name).! !

!RPackageDynamicTest methodsFor: 'test move' stamp: 'TestRunner 12/3/2009 16:37'!
testMoveClassDefinition
	"self debug: #testMoveClassDefinition"
	
	
	self deny: (p2 includesOrTouchesClass: a1).
	self deny: (p2 includesClass: a1).
	self assert: (p1 definedClasses  includes: a1 name). 
	
	p1 moveClassDefinition: a1 to: p2.
	self deny: (p1 includesClass: a1).
	
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	self assert: (p1 includesOrTouchesClass: a1).
	"since the method A1DefinedInP1>>methodDefinedInP1 did not move"
	
	self deny: (p1 definedClasses  includes: a1). 

	
	! !

!RPackageDynamicTest methodsFor: 'test move' stamp: 'stephane.ducasse 8/12/2009 10:22'!
testMoveClassDefinitionNotDefinedInSourcePackage
	"self debug: #testMoveClassDefinitionNotDefinedInSourcePackage"
	
	
	self assert: (p2 includesOrTouchesClass: a2).
	self assert: (p2 includesClass: a2).
	self deny: (p1 definedClasses includes: a2). 
	
	p1 moveClassDefinition: a2 to: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: a2).
	
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self assert: (p1 includesOrTouchesClass: a2).
	"since moveClassDefinition just move the class definition and not its methods"
	
	! !

!RPackageDynamicTest methodsFor: 'test move' stamp: 'TestRunner 12/3/2009 10:25'!
testMoveMethod
	"self debug: #testMoveMethod"
	
	
	self deny: (p2 includesOrTouchesClass: a1).
	p1 moveMethod: (a1 >> #methodDefinedInP1) to: p2.
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	
	self assert: (p2 includesMethod: #methodDefinedInP1 ofClass: a1).
	self assert: (p2 includesOrTouchesClass: a1).
	! !

!RPackageDynamicTest methodsFor: 'test move' stamp: 'TestRunner 12/3/2009 10:34'!
testMoveMethodExtensionToItsClassHomePackage
	"self debug: #testMoveMethodExtensionToItsClassHomePackage"
	
	p1 moveMethod: (a2 >> #methodDefinedInP1) to: p2.
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	
	self assert: (p2 includesMethod: #methodDefinedInP1 ofClass: a2).
	


	
	! !

!RPackageDynamicTest methodsFor: 'test accessing' stamp: 'TestRunner 10/29/2009 20:39'!
testPackagesOfClass
	"since a class can be extended by several packages, we want the complete list of packages that extend
	the class"
	
	"self debug: #testPackagesOfClass"
	
	"P1 
		A1DefinedInP1>>methodDefinedInP1
		B1DefinedInP1	
		A2DefinedInP2>>methodDefinedInP1  (was *P2)
	
	P2
		A2DefinedInP2>>methodDefinedInP2
		B2DefinedInB2	

	P3
		A3DefinedInP3	
		A2DefinedInP2>>methodDefinedInP3  (was *P2*)"
		
	| packages |
	self assert: a2 methodDict keys size = 3.
	packages := a2 packages.
	"a2 is extended by p1 and p3"
	self assert: packages size = 3. 
	self assert: (packages includes: p1).
	self assert: (packages includes: p2).
	self assert: (packages includes: p3).
	
	
	
	
	
	
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'stephane.ducasse 8/12/2009 00:00'!
testRemoveClassFromAllPackages
	"we remove a class we check that it is not in the package anymore and that other class extensions
	are removed from the other packages"
	"self debug: #testRemoveClassFromAllPackages"
	
	p2 removeClassFromAllPackages: a2.
	self deny: (p2 includesClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self deny: (p3 includesMethod: #methodDefinedInP3 ofClass: a2).
	self assert: (p1 includesClass: a1).
	self assert: (p1 includesClass: b1).
	self assert: (p2 includesClass: b2).
	self assert: (p3 includesClass: a3).! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 9/11/2009 15:36'!
testRemoveClassFromSystem
	"we remove a class we check that it is not in the package anymore and that other class extensions
	are removed from the other packages"
	"self debug: #testRemoveClassFromSystem"
	
	| a2name |
	a2name := a2 name.
	self shouldnt: [ Smalltalk at: a2name asSymbol] raise: Error.
	p2 removeClassFromSystem: a2.
	self should: [ Smalltalk at: a2name asSymbol] raise: Error.
	
	"copy and paste from testRemoveClassFromAllPackages"
	self deny: (p2 includesClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self deny: (p3 includesMethod: #methodDefinedInP3 ofClass: a2).
	self assert: (p1 includesClass: a1).
	self assert: (p1 includesClass: b1).
	self assert: (p2 includesClass: b2).
	self assert: (p3 includesClass: a3).! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:41'!
testRemoveExtensionMethod
	"self debug: #testRemoveExtensionMethod"
	
"P1 
	A1DefinedInP1>>methodDefinedInP1
	B1DefinedInP1	
	A2DefinedInP2>>methodDefinedInP1  (= *P2)
	
P2
	A2DefinedInP2>>methodDefinedInP2
	B2DefinedInB2	

P3
	A3DefinedInP3	
	A2DefinedInP2>>methodDefinedInP3  (= *P2*)"
	
	p1 removeMethod:  (a2 >> #methodDefinedInP1).
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:41'!
testRemoveExtensionMethodDoesNotRemoveFromClass
	"self debug: #testRemoveExtensionMethodDoesNotRemoveFromClass"
	
	p1 removeMethod: (a2>>#methodDefinedInP1).
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self assert: (a2 selectors includes: #methodDefinedInP1)
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:42'!
testRemoveExtensionMethodFromSystemRemovesItFromClass
	"self debug: #testRemoveExtensionMethodFromSystemRemovesItFromClass"
	
	p1 removeMethodFromSystem: a2 >>#methodDefinedInP1.
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	self deny: (a2 selectors includes: #methodDefinedInP1)
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:42'!
testRemoveMethod
	"self debug: #testRemoveMethod"
	
	p1 removeMethod: a1 >> #methodDefinedInP1.
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:42'!
testRemoveMethodDoesNotRemoveFromClass
	"we remove a method so it is not in the package but also not from the class anymore"
	"self debug: #testRemoveMethodDoesNotRemoveFromClass"
	
	p1 removeMethod:  a1 >> #methodDefinedInP1.
	self deny: (p1 includesMethod: #methodDefinedInP1 ofClass: a1).
	self assert: (a1 selectors includes: #methodDefinedInP1)! !

!RPackageDynamicTest methodsFor: 'test addition removal' stamp: 'StephaneDucasse 12/3/2009 15:42'!
testRemoveShouldRaiseErrorWhenMethodIsNotInPackage
	"self debug: #testRemoveShouldRaiseErrorWhenMethodIsNotInPackage"
	
	self should: [p2 removeMethod: a2>>#methodDefinedZork] raise: Error
	
! !

!RPackageDynamicTest methodsFor: 'test creation' stamp: 'stephane.ducasse 8/12/2009 10:26'!
testStartingSituation
	"self debug: #testStartingSituation"
	
	self deny: (p2 includesClass: b1).
	self assert: (p2 includesClass: b2).
	"a locally defined class not extended by other packages"
	
	self assert: (p2 includesClass: a2).  
	"a locally defined class extended by other packages"
	
	self assert: (p1 includesOrTouchesClass: a2).
	self deny: (p1 includesClass: a2).! !

!RPackageDynamicTest methodsFor: 'test creation' stamp: 'StephaneDucasse 12/3/2009 17:26'!
testStartingSituationNoClassHoldInInternalStructure
	"self debug: #testStartingSituationNoClassHoldInInternalStructure"
	
	self assert: ((p2 instVarNamed: 'methodDict') keys allSatisfy:[:each | each isSymbol])
	! !

RPackageTest subclass: #RPackageOrganizerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage'!

!RPackageOrganizerTest methodsFor: 'setup' stamp: 'stephane.ducasse 8/10/2009 11:41'!
setUp

	super setUp.
	RMockPackageOrganizer reset.
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'stephane.ducasse 8/10/2009 11:41'!
testEmpty
	"self debug: #testEmpty"

	self assert: RMockPackageOrganizer default packageNames isEmpty
	! !

!RPackageOrganizerTest methodsFor: 'tests' stamp: 'stephane.ducasse 8/10/2009 11:50'!
testRegisteredIsThere
	"self debug: #testRegisteredIsThere"

	| p1 p2 p3 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.
	
	RMockPackageOrganizer default registerPackage: p1.
	RMockPackageOrganizer default registerPackage: p2.
	RMockPackageOrganizer default registerPackage: p3.
	self assert: RMockPackageOrganizer default packageNames size = 3.
	RMockPackageOrganizer default unregisterPackage: p3.
	self assert: RMockPackageOrganizer default packageNames size = 2.
	! !

!RPackageTest class methodsFor: 'history' stamp: 'stephane.ducasse 7/10/2009 09:11'!
lastStoredRun
	^ ((Dictionary new) add: (#passed->((Set new) add: #testMethodCreation; add: #testDefinedLocalClasses; add: #testIncludesMethodOfClass; add: #testClassAddition; add: #testClassAdditionNoDuplicate; add: #testMethodCreationMethodRefAsEntry; add: #testClassExtensionClasses; add: #testIncludesMethodOfClassInPresenceOfOtherPackageExtensions; add: #testClassExtensionMethods; add: #testClassExtensionExtensionMethods; add: #testCheckUniqueClassInPackage; yourself)); add: (#timeStamp->'10 July 2009 9:11:38 am' asTimeStamp); add: (#failures->((Set new))); add: (#errors->((Set new) add: #testClassExtensionExtendedClasses; yourself)); yourself)! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 6/1/2009 22:28'!
createNewClassNamed: aName
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'RPackageTest'.
	createdClasses add: cls.
	^ cls! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 6/1/2009 22:28'!
createNewClassNamed: aName inCategory: cat
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: cat.
	createdClasses add: cls.
	^ cls! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 8/12/2009 10:06'!
createNewClassNamed: aName inPackage: p
	
	| cls |
	cls := self createNewClassNamed: aName.
	p addClassDefinition: cls.
	^ cls! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 6/24/2009 11:06'!
createNewPackageNamed: aName

	| pack |
	self removePackageIfExist: aName.
	pack := RPackage named: aName.
	createdPackages add: pack.
	^ pack! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 6/1/2009 22:28'!
defaultRepository

	^ defaultRepository ifNil: [defaultRepository := MCCacheRepository default]! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 6/1/2009 22:28'!
removeClassNamedIfExists: aClassNameSymbol

	Smalltalk at: aClassNameSymbol asSymbol ifPresent: [:c| c removeFromSystem]! !

!RPackageTest methodsFor: 'utility' stamp: 'stephane.ducasse 7/20/2009 14:43'!
removePackageIfExist: aName

	self flag: #toDo.
	"
	(Package exist: aName) 
		ifTrue: [(Package named: aName) removeFromSystem]."
! !

!RPackageTest methodsFor: 'setup' stamp: 'StephaneDucasse 12/3/2009 21:46'!
setUp

	RPackage packageOrganizerClass: RMockPackageOrganizer.
	createdClasses := OrderedCollection new.
	createdPackages := OrderedCollection new.! !

!RPackageTest methodsFor: 'setup' stamp: 'StephaneDucasse 12/3/2009 21:49'!
tearDown

	createdPackages do: [:p| p removeFromSystem].
	(createdClasses reject: [:c| c isObsolete])
		do: [:cls| 
				RMockPackageOrganizer default unregisterClass: cls.
				cls removeFromSystem.				
				].
	RPackage packageOrganizerClass: RPackageOrganizer! !

!RPackageTest methodsFor: 'test extensions' stamp: 'TestRunner 12/2/2009 21:35'!
testAccessMethodOfClass
	"self debug: #testAccessMethodOfClass"
	
	| p1 p2 p3 a2  mr |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3).
		
	self assert: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	
	self deny: (p2 includesMethod: #methodDefinedInP3 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP1 ofClass: a2).
	
	
	
	
	
	! !

!RPackageTest methodsFor: 'tests class creation' stamp: 'stephane.ducasse 8/12/2009 10:06'!
testCheckUniqueClassInPackage
	"self debug: #testCheckUniqueClassInPackage"

	| p a1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
		
	self flag: #AutoPackageStep.
	self assert: p classes size = 0.
	p addClassDefinition: a1.
	self assert: p classes size = 1.
	p addClassDefinition: a1.
	self assert: p classes size = 1.

! !

!RPackageTest methodsFor: 'tests class creation' stamp: 'stephane.ducasse 8/12/2009 10:07'!
testCheckUniqueClassInPackageWithAddClass
	"self debug: #testCheckUniqueClassInPackageWithAddClass"

	| p a1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
		
	self flag: #AutoPackageStep.
	self assert: p classes size = 0.
	p addClass: a1.
	self assert: p classes size = 1.
	p addClass: a1.
	self assert: p classes size = 1.

! !

!RPackageTest methodsFor: 'tests class creation' stamp: 'TestRunner 12/3/2009 16:05'!
testClassAddition
	"self debug: #testClassAddition"

	| p a1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	self assert: p classes size = 0.
	p addClassDefinition: a1.
	self assert: p classes size = 1.
	self assert: (p includesClass: a1).
	self assert: (p includesClass: a1 class)

! !

!RPackageTest methodsFor: 'tests class creation' stamp: 'stephane.ducasse 8/12/2009 10:07'!
testClassAdditionNoDuplicate
	"self debug: #testClassAdditionNoDuplicate"

	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPackageP1 inCategory: 'P1'.

	self flag: #AutoPackageStep. "will changed when the class will be automatically added to the package"
	self assert: p classes size = 0.
	p addClassDefinition: a1.
	self assert: p classes size = 1.
	b1 := self createNewClassNamed: #B1InPackageP1 inCategory: 'P1'.
	p addClassDefinition: a1.
	self assert: p classes size = 1.
	p addClassDefinition: b1.
	self assert: p classes size = 2.
! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 12/3/2009 15:38'!
testClassExtensionClasses
	"self debug: #testClassExtensionClasses"
	
	| p1 p2  a2 b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: b2).
	self assert: (p2 includesClass: b2).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addSelector: #methodDefinedInP1 ofClass: a2.
	
	self assert: p1 extensionMethods size = 1.
	self assert: p1 extensionClasses size = 1.
	self deny: (p1 includesClass: a2).
	"method extension class are not included in packages"
	
	b2 compile: 'firstMethodInB2PackagedInP1 ^ 1'.
	p1 addSelector: #firstMethodInB2PackagedInP1 ofClass: b2.
	
	self assert: p1 extensionMethods size = 2.
	self assert: p1 extensionClasses size = 2.
	self deny: (p1 includesClass: b2).
	
	b2 compileSilently: 'secondMethodInB2PackagedInP1 ^ 2'.
	p1 addSelector: #secondMethodInB2PackagedInP1 ofClass: b2.
	
	self assert: p1 extensionMethods size = 3.
	self assert: p1 extensionClasses size = 2.
	self deny: (p1 includesClass: b2).! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 12/3/2009 15:37'!
testClassExtensionClassesWithCompiledMethod
	"self debug: #testClassExtensionClassesWithCompiledMethod"
	
	| p1 p2  a2 b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: b2).
	self assert: (p2 includesClass: b2).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1).
	
	self assert: p1 extensionMethods size = 1.
	self assert: p1 extensionClasses size = 1.
	self deny: (p1 includesClass: a2).
	"method extension class are not included in packages"
	
	b2 compile: 'firstMethodInB2PackagedInP1 ^ 1'.
	p1 addMethod: (b2>>#firstMethodInB2PackagedInP1).
	
	self assert: p1 extensionMethods size = 2.
	self assert: p1 extensionClasses size = 2.
	self deny: (p1 includesClass: b2).
	
	b2 compileSilently: 'secondMethodInB2PackagedInP1 ^ 2'.
	p1 addMethod: (b2>>#secondMethodInB2PackagedInP1).
	
	self assert: p1 extensionMethods size = 3.
	self assert: p1 extensionClasses size = 2.
	self deny: (p1 includesClass: b2).! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 10/29/2009 20:37'!
testClassExtensionExtensionMethods
	"self debug: #testClassExtensionExtensionMethods"
	
	| p1 p2 a2  b2 |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	b2 := self createNewClassNamed: #B2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1) methodReference.
	
	self assert: p1 extensionMethods size = 1.
	self deny: (p1 includesClass: a2).
	"method extension class are not included in packages"
	
	b2 compileSilently: 'firstMethodInB2PackagedInP1 ^ 1'.
	p1 addMethod: (b2>>#firstMethodInB2PackagedInP1) methodReference.
	self assert: p1 extensionMethods size = 2.
	! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 10/29/2009 20:37'!
testClassExtensionMethods
	"self debug: #testClassExtensionMethods"
	
	| p1 p2 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	self deny: (p1 includesClass: a2).
	self assert: (p2 includesClass: a2).
	
	a2 compileSilently: 'methodPackagedInP1 ^ #methodPackagedInP1'.
	p1 addMethod: (a2>>#methodPackagedInP1) methodReference.
	
	self assert: p1 extensionMethods size = 1.
	self deny: (p1 includesClass: a2).
	self assert: (p1 includesOrTouchesClass: a2).
	self assert: (p2 includesClass: a2).! !

!RPackageTest methodsFor: 'tests class creation' stamp: 'stephane.ducasse 8/12/2009 10:14'!
testDefinedLocalClasses
	"self debug: #testDefinedLocalClasses"
	"definedClasses returns the list of the class defined in a package by opposition to all classes 
	which are the extended classes and the defined ones"
	
	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inCategory: 'P1'.
	b1 := self createNewClassNamed: #B1InPAckageP1 inCategory: 'P1'.

	self flag: #AutoPackageStep. "will changed when the class will be automatically added to the package"
	self assert: p definedClasses size = 0.
	p addClassDefinition: a1.
	p addClassDefinition: b1.
	self assert: p classes size = 2.
	self assert: (p includesClass: a1).
	self assert: (p includesClass: b1)


! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 10/29/2009 20:37'!
testIncludesMethodOfClass
	"self debug: #testIncludesMethodOfClass"
	
	| p1 p2 p3 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2) methodReference.
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1) methodReference.
	
	a2 compileSilently: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3) methodReference.
		
	self assert: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP3 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP1 ofClass: a2).
	! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 10/29/2009 20:38'!
testIncludesMethodOfClassInPresenceOfOtherPackageExtensions
	"self debug: #testIncludesMethodOfClassInPresenceOfOtherPackageExtensions"
	
	| p1 p2 p3 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2) methodReference.
	self assert: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1) methodReference.
	self assert: (p1 includesMethod: #methodDefinedInP1 ofClass: a2).
	
	a2 compile: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3) methodReference.
	self assert: (p3 includesMethod: #methodDefinedInP3 ofClass: a2).
		
	self assert: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP3 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP1 ofClass: a2).
	! !

!RPackageTest methodsFor: 'tests method creation' stamp: 'TestRunner 12/2/2009 21:43'!
testMethodCreation
	"self debug: #testMethodCreation"

	| p1 a1 methodRef |
	p1 := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1DefinedInP1 inPackage: p1.
	a1 compileSilently: 'foo ^ 10'.
	
	methodRef := (a1>>#foo) methodReference.
	p1 addMethod: methodRef.
	self assert: (p1 includesMethod: #foo ofClass: a1).
	! !

!RPackageTest methodsFor: 'tests method creation' stamp: 'TestRunner 12/2/2009 21:52'!
testMethodCreationMethodRefAsEntry
	"self debug: #testMethodCreationMethodRefAsEntry"

	| p1 a1 methodRef |
	p1 := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1DefinedInP1 inPackage: p1.
	a1 compileSilently: 'foo ^ 10'.
	
	methodRef := MethodReference class: a1 selector: #foo.
	p1 addMethod: methodRef.
	self assert: (p1 includesMethod: #foo ofClass: a1).
! !

!RPackageTest methodsFor: 'test class navigation' stamp: 'StephaneDucasse 10/29/2009 20:38'!
testPackageOfClassForClassesNotDefinedInPackageButJustExtendingIt
	"self debug: #testPackageOfClassForClassesNotDefinedInPackageButJustExtendingIt"
	
	
	| p1 p2 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1) methodReference.
	
	self assert: a2 package = p2.
	p1 extensionMethods do: 
		[:each | self deny: each class package = p1 ].
	"the package of a class which is extended inside a package p, is not p
	but the package where the class was defined"
	
	
	! !

!RPackageTest methodsFor: 'test class navigation' stamp: 'StephaneDucasse 10/27/2009 21:37'!
testPackageOfClassForDefinedClasses
	"self debug: #testPackageOfClassForDefinedClasses"
	
	
	| p a1 b1 |
	p := self createNewPackageNamed: 'P1'.
	a1 := self createNewClassNamed: #A1InPAckageP1 inPackage: p.
	b1 := self createNewClassNamed: #B1InPAckageP1 inPackage: p.

	self assert: a1 package = p.
	self assert: b1 package = p.


! !

!RPackageTest methodsFor: 'test extensions' stamp: 'StephaneDucasse 10/29/2009 20:38'!
testRemoveMethodOfClass
	"self debug: #testRemoveMethodOfClasstestRemoveMethodOfClass"
	
	| p1 p2 p3 a2  |
	p1 := self createNewPackageNamed: 'P1'.
	p2 := self createNewPackageNamed: 'P2'.
	p3 := self createNewPackageNamed: 'P3'.


	

	a2 := self createNewClassNamed: #A2InPackageP2 inPackage: p2.
	a2 compileSilently: 'methodDefinedInP2 ^ #methodDefinedInP2'.
	p2 addMethod: (a2>>#methodDefinedInP2) methodReference.
	
	a2 compileSilently: 'methodDefinedInP1 ^ #methodDefinedInP1'.
	p1 addMethod: (a2>>#methodDefinedInP1) methodReference.
	
	a2 compile: 'methodDefinedInP3 ^ #methodDefinedInP3'.
	p3 addMethod: (a2>>#methodDefinedInP3) methodReference.
		
	self assert: (p2 includesMethod: #methodDefinedInP2 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP3 ofClass: a2).
	self deny: (p2 includesMethod: #methodDefinedInP1 ofClass: a2).
	! !
RPackage initialize!
